#! perl

=head1 NAME

RSSCache - Main module of the RSS cache.

=head1 SYNOPSIS

	use RSSCache;
	RSSCache->start();

=head1 DESCRIPTION

This program will work like a caching RSS proxy. You can access RSS, RDF and Atom feeds
through this cache like this:

	http://cache.server:port/http://some/feed.xml

The caching server will download the feed and cache it. Nothing special yet.

The advantage of this cache is that it keeps the feed items inside the cache for
a specified amount of time (e.g. one week) instead of just returning the last 20
items or so. This is very handy if you have very busy feeds (>20 items a day or so)
and you don't want to keep your RSS reader running all the time so that you don't
miss any item.

An additional big advantage is gained through the client identification. You can
get a client id (CID) from the RSSCache through this URL:

	http://cache.server:port/generate_cid

The server will return a client id like 'k4JOgtlf'. When using this CID inside
your feed requests:

	http://cache.server:port/cid_k4JOgtlf/http://some/feed.xml
	
The RSSCache knows who you are and will only serve feed items that you haven't
downloaded yet. This will save lots of bandwith for you, the RSSCache and the
feed distributor itself. Great isn't it?

Note: This program massively uses Perl ithreads which may have negative effects
on memory usage and performance. You will need Perl 5.6 or higher compiled with
threads support.

=head1 CONFIGURATION

This program is configured by a configuration file called 'RSSCache.settings',
which should reside in the same directory as the program itself.

The supplied default configuration file should be self-explaining.

=cut

package RSSCache;

use strict;
use warnings;

use threads;
use threads::shared;
use Config;

use RSSCache::Log;
use RSSCache::HTTPServer;
use RSSCache::Poller;
use RSSCache::Scheduler;
use RSSCache::Settings;
use RSSCache::Admin::Console;
use RSSCache::Backend::Memory;

our $VERSION = 0.2;

=head1 METHODS

=head2 start

Some kind of a constructor.

You should not create multiple objects of this class as it must use package scoped
variables for the threading model that would be accidentially shared between all
instances of this class. So just start it once as stated in the L</SYNOPSIS>.

=cut
sub start {
	my ($class) = @_;
	my $self = bless {}, $class;
	
	#check for thread support
	$Config{useithreads} or die "Recompile your Perl with -Dusethreads as this program uses Perl ithreads.\nBye\n";
	
	#save starttime
	$RSSCache::StartTime = time();
	
	#initialize random number generator
	srand(time() ^ ($$ + ($$ << 15)));
	
	#create globally used objects
	$RSSCache::Settings = RSSCache::Settings->new();
	$RSSCache::Log      = RSSCache::Log->new();
	$RSSCache::Log->info("$0/$VERSION started.");
	$RSSCache::Poller   = RSSCache::Poller->new();
	$RSSCache::Backend  = RSSCache::Backend::Memory->new();
	
	#load initial feeds, poll all feeds.
	$RSSCache::Log->notice(__PACKAGE__."->start(): Loading initial feeds...");
	$RSSCache::Backend->load_initial_feeds();
	$RSSCache::Log->notice(__PACKAGE__."->start(): Polling all loaded feeds...");
	$RSSCache::Poller->poll();
	
	#create permanently running threads
	my $scheduler = RSSCache::Scheduler->new();
	my $listener  = RSSCache::HTTPServer->new();
	my $console   = RSSCache::Admin::Console->new();
	
	#wait for threads to finish (although they won't ever finish...)
	$console->join();
	$scheduler->join();
	$listener->join();
}
# /start

=head2 stop

This method exits this program

=cut
sub stop {
	$RSSCache::Log->info("Shutting down...");
	#save client ids and feeds here
	$RSSCache::Backend->store_data();
	
	exit 0;
}
# /stop

=head1 AUTHOR

Copyright 2006 Thomas Wittek (mail at gedankenkonstrukt dot de). All rights reserved. 

=head1 LICENsE

This document is free software.
It is distributed under the same terms as Perl itself.

=head1 TODO

=over

=item * TODO: Testing of deletion of old items.

=item * TODO: thread creation seems to be very slow. speed up creation or keep up threeads.

=item * TODO: Term::ReadLine and Term::ANSIColor don't work under Win32 (ActivePerl)

=item * TODO: Term::ANSIColor outputs seem to be much slower than plain text outputs

=item * TODO: Test if Feed-Clients have problems with empty feeds

=item * FEATURE: catch SIGKILL and evtl.

=item * FEATURE: support multiple channels in one rss-feed

=item * FEATURE: Enable logging to file

=back

=head1 SEE ALSO

L<RSSCache::Backend>, L<RSSCache::HTTPServer>, L<RSSCache::Log>, L<RSSCache::Poller>, L<RSSCache::Scheduler>, L<RSSCache::Settings>, L<RSSCache::Admin::Console>, L<RSSCache::Backend::Memory>

=cut

1;
