#! perl

=head1 NAME

RSSCache::Backend - Base class for all backend implementations

=head1 SYNOPSIS

	use RSSCache::Backend;
	@ISA = qw/RSSCache::Backend/;

=head1 DESCRIPTION

This is the base class for all backend implementation classes.

Currently there exists only one backend implementation (L<RSSCache::Backend::Memory>)
but it may be followed by others (like DBI).

When creating an own backend implementation, you must overwrite the methods of
this base class, that will print out an error message otherwise.

=cut

package RSSCache::Backend;

use strict;
use warnings;

=head1 METHODS

=head2 new

Constructor.

You should inherit or call this constructor to set the default settings:

	sub new {
		return bless RSSCache::Backend->new(), $class;
	}

=cut
sub new {
	my ($class) = @_;
	
	#create object
	my $self = bless {}, $class;
	
	#set default settings
	$RSSCache::Settings->set('backend/cache_duration',       '7d') unless defined $RSSCache::Settings->get('backend/cache_duration');
	$RSSCache::Settings->set('backend/max_offtime',          '2d') unless defined $RSSCache::Settings->get('backend/max_offtime');
	$RSSCache::Settings->set('backend/data_file', 'RSSCache.data') unless defined $RSSCache::Settings->get('backend/data_file');
	
	return $self;
}
# /new

=head2 feeds

Returns an array of the URLs of all feeds that are currently cached.

=cut
sub feeds {
	my ($self) = @_;
	$RSSCache::Log->error(__PACKAGE__."->feeds() not overridden by implementation class!");
	return ();
}
# /feeds

=head2 feed_loaded

Returns true if the feed with the specified URL has already been added to the
backend store.

=head3 Parameters:

=over

=item * $url - The URL of the feed

=back

=cut
sub feed_loaded {
	my ($self, $url, $cid) = @_;
	$RSSCache::Log->error(__PACKAGE__."->feed_loaded() not overridden by implementation class!");
	return undef;
}
# /feed_loaded

=head2 feed

Returns the content of one feed specified by its URL.

=head3 Parameters:

=over

=item * $url - The URL of the feed, which content should be returned.

=item * $cid - The client id (CID) of the requesting client.

If defined, your backend implementation should only return the feed items that
the client hasn't downloaded yet. So your backend has to save the date of the
last download for each CID and should only return the feed items younger than
this date. If there are no new feeds, your implementation should/may return
an empty string to save even more traffic.

Unless defined (e.g. = undef) your implementation should return the whole feed.

=back

=cut
sub feed {
	my ($self, $url, $cid) = @_;
	$RSSCache::Log->error(__PACKAGE__."->feed() not overridden by implementation class!");
	return undef;
}
# /feed

=head2 item_count

Returns the number of items of a specified feed.

=head3 Parameters:

=over

=item * $url - The URL of the feed

=back

=cut
sub item_count {
	my ($self, $url) = @_;
	$RSSCache::Log->error(__PACKAGE__."->item_count() not overridden by implementation class!");
	return undef;
}
# /item_count

=head2 feed_error

Will be called by the feed poller when a feed couldn't be downloaded.

The backend implementation might store the date of the last successful download
and remove a feed from the cache if the last successful download was a too long
time ago when an error occurs.

=head3 Parameters:

=over

=item * $url - The URL of the feed, which couldn't be downloaded.

=back

=cut
sub feed_error {
	my ($self, $url) = @_;
	$RSSCache::Log->error(__PACKAGE__."->feed_error() not overridden by implementation class!");
	return undef;
}
# /feed_error

=head2 add_or_update_feed

Will add or update a feed specified by its URL and its current content.

Within this method the backend implementation should add a new feed/new items to
its cache. You might want to use L</parse_feed> to parse the feed content.

This method will be called by the L<RSSCache::Poller>.

=head3 Parameters:

=over

=item * $url - The URL of the feed.

=item * $date - The date of the feed, which has been downloaded

=back

=cut
sub add_or_update_feed {
	my ($self, $url, $data) = @_;
	$RSSCache::Log->error(__PACKAGE__."->add_or_update_feed() not overridden by implementation class!");
	return 0;
}
# /add_or_update_feed

=head2 delete_feed

Deletes the specified feed.

=head3 Parameters:

=over

=item * $url - The URL of the feed.

=back

=cut
sub delete_feed {
	my ($self, $url) = @_;
	$RSSCache::Log->error(__PACKAGE__."->delete_feed() not overridden by implementation class!");
	return 0;
}
# /delete_feed

=head2 clear

Clears the whole backend cache (i.e. deletes all cached feeds).

=cut
sub clear {
	my ($self) = @_;
	$RSSCache::Log->error(__PACKAGE__."->clear() not overridden by implementation class!");
	return 0;
}
# /clear

=head2 parse_feed

Parses a given feed and returns an hashref:

	{
		start => 'eveything before the first item',
		end => 'everything after the last item',
		items => [
			'<item> ... </item>',
			...
		]
	}

Can handle RSS, RDF and Atom feeds.

=head3 Parameters:

=over

=item * $url - The URL of the feed.

=item * $date - The feed''s content.

=back

=cut
sub parse_feed {
	my ($self, $url, $data) = @_;
	if ($data =~ /<rdf[^>]*?>.*<\/rdf[^>]*?>/so) {
		#rdf feed
		return $self->_parse_rdf($url, $data);
	} elsif ($data =~ /<rss[^>]*?>.*<\/rss>/so) {
		#rss feed
		return $self->_parse_rss($url, $data);
	} elsif ($data =~ /<feed[^>]*?>.*<\/feed>/so) {
		#atom feed
		return $self->_parse_atom($url, $data);
	} else {
		#unsupported feed
		$RSSCache::Log->error(__PACKAGE__."->parse_feed(): The feed '$url' has an invalid feed type. Currently only RDF, RSS and ATOM are supported.");
		return undef;
	}
}
# /parse_feed

sub _parse_rss {
	my ($self, $url, $data) = @_;
	if ($data =~ /(<rss[^>]*?>.*?)(<item\W.*<\/item>)(.*?<\/rss>)/so) {
		my ($feedstart, $items, $feedend) = ($1, $2, $3);
		my @items = ($items =~ /(<item\W.*?<\/item>)/gso);
		return { start => $feedstart, end => $feedend, items => \@items };
	} else {
		$RSSCache::Log->error(__PACKAGE__."->parse_rss(): Invalid or empty RSS feed '$url'");
		return undef;
	}
}
# /_parse_rss

sub _parse_atom {
	my ($self, $url, $data) = @_;
	if ($data =~ /(<feed[^>]*?>.*?)(<entry.*<\/entry>)(.*?<\/feed>)/so) {
		my ($feedstart, $items, $feedend) = ($1, $2, $3);
		my @items = ($items =~ /(<entry.*?<\/entry>)/gso);
		return { start => $feedstart, end => $feedend, items => \@items };
	} else {
		$RSSCache::Log->error(__PACKAGE__."->parse_rss(): Invalid or empty ATOM feed '$url'");
		return undef;
	}
}
# /_parse_atom

sub _parse_rdf {
	my ($self, $url, $data) = @_;
	if ($data =~ /(<rdf(?::RDF)?[^>]*?>.*?)(<item\W.*<\/item>)(.*?<\/rdf(?::RDF)?>)/so) {
		my ($feedstart, $items, $feedend) = ($1, $2, $3);
		my @items = ($items =~ /(<item\W.*?<\/item>)/gso);
		$RSSCache::Log->debug("start: $feedstart");
		foreach my $item (@items) {
			$RSSCache::Log->debug("item: $item");
		}
		$RSSCache::Log->debug("end: $feedend");
		return { start => $feedstart, end => $feedend, items => \@items };
	} else {
		$RSSCache::Log->error(__PACKAGE__."->parse_rdf(): Invalid or empty RDF feed '$url'");
		return undef;
	}
}
# /_parse_rdf

=head2 configtime_to_unixtime

Converts a duration definition in the format "1y 2m 3d" (= 1 year, 2 months, 3 days)
into seconds and returns it.

This is useful when you want to allow the user to define durations within the
config file.

=head3 Parameters:

=over

=item * $configtime - The duration in the format "1y 2m 3d"

=back

=cut
sub configtime_to_unixtime {
	my ($self, $configtime) = @_;
	
	my @offsets = split /\s+/, $configtime;
	my @diff = ((0) x 6);
	my %letter_to_index = (y => 0, m => 1, d => 2);
	foreach my $offset (@offsets) {
		$offset =~ /^(\d+)(.)$/;
		if ($1 and $2 and exists($letter_to_index{lc($2)})) {
			$diff[$letter_to_index{lc($2)}] += $1;
		}
	}
	#convert date into unix time format and return this date
	return $diff[0] * 31_536_000 + $diff[1] * 2_592_000 + $diff[2] * 86_400;
}
# /configtime_to_unixtime 

=head2 load_initial_feed

Will download and cache the initial feeds that have been defined in the
configuration file.

Will be called on startup.

=cut
sub load_initial_feeds {
	my ($self) = @_;
	
	#add initial feeds
	if (defined (my $init_feeds = $RSSCache::Settings->get('backend/initial_feeds'))) {
		#only poll feeds not already loaded.
		my @feeds = map { $self->feed_loaded($_) ? () : $_ } split /\s+/, $init_feeds;
		$RSSCache::Poller->poll(@feeds) if @feeds;
	}
}
# /load_initial_feeds

=head2 generate_client_id

Generates and returns an unique 8 char ([a-zA-Z0-6]) long client ID.

Your backend implementation doesn't have to overwrite this method, so it might
be inherited.

=cut
sub generate_client_id {
	my ($self) = @_;
	
	my $cid = '';
	my $length = 8;
	do {
		#0-9: 48-57
		#A-Z: 65-90
		#a-z: 97-122
		for (my $i = 1; $i <= $length; $i++) {
			my $num .= int(rand(62));
			if ($num < 10) {
				$cid .= chr($num + 48);
			} elsif ($num < (36)) {
				$cid .= chr($num + 55);
			} else {
				$cid .= chr($num + 61);
			}
		}
	} until (not $self->client_id_exists($cid));
	
	$self->create_client_id($cid);
	
	return $cid;
}
# /generate_client_id

=head2 client_id_exists

Returns true, if a given client id (CID) exists.

Will be used by the method generate_client_id to avoid creating the same CID twice.

See also L</create_client_id>.

=head3 Parameters:

=over

=item * $cid - The client id

=back

=cut
sub client_id_exists {
	my ($self, $cid) = @_;
	$RSSCache::Log->error(__PACKAGE__."->client_id_exists() not overridden by implementation class!");
	return 0;
}
# /client_id_exists

=head2 set_client_id

Adds a new client id (CID) to the data store.

Will be called by L</generate_client_id> right after a new CID has been created.

The CID has to be stored explicitly to avoid creating the same CID twice.

=head3 Parameters:

=over

=item * $cid - The client id

=back

=cut
sub create_client_id {
	my ($self, $cid) = @_;
	$RSSCache::Log->error(__PACKAGE__."->create_client_id() not overridden by implementation class!");
	return 0;
}
# /create_client_id

=head2 store_data

Stores the current date to a persistent medium (e.g. hard disk). Will be called
at shutdown. May be scheduled regularly.

=cut
sub store_data {
	my ($self) = @_;
	$RSSCache::Log->error(__PACKAGE__."->store_data() not overridden by implementation class!");
	return 0;
}
# /store_data

=head1 AUTHOR

Copyright 2006 Thomas Wittek (mail at gedankenkonstrukt dot de). All rights reserved. 

=head1 LICENsE

This document is free software.
It is distributed under the same terms as Perl itself.

=head1 SEE ALSO

L<RSSCache>, L<RSSCache::Backend::Memory>

=cut

1;
