#! perl

=head1 NAME

RSSCache::HTTPServer - Serves the cached feeds.

=head1 SYNOPSIS

	my $listener = RSSCache::HTTPServer->new();

=head1 DESCRIPTION

This class will start a thread listening for HTTP requests on the port specified
in the configuration file.

It will serve each feed request in a new thread. If a requested feed hasn't been
cached yet, it will be downloaded and added to the cache.

=cut

package RSSCache::HTTPServer;

use strict;
use warnings;

use threads;

use Encode;
use IO::Socket;
use HTTP::Parser;
use HTTP::Response;

use constant HTTP_HEADER_COMPLETE => 0;

=head1 METHODS

=head2 new

Constructor.

Will start the listening thread.

=cut
sub new {
	my ($class) = @_;
	
	#create object
	my $self = bless {}, $class;
	
	#set default settings
	$RSSCache::Settings->set('server/port',             2342) unless defined $RSSCache::Settings->get('server/port');
	$RSSCache::Settings->set('server/encoding', 'iso-8859-1') unless defined $RSSCache::Settings->get('server/encoding');
	
	return threads->new(\&listen, $self);
}
# /new

#starts the listening thread that will start new threads to handle each connection
sub listen {
	my ($self) = @_;
	
	my $port = $RSSCache::Settings->get('server/port');
	my $listening_socket = new IO::Socket::INET(LocalPort => $port, Proto => 'tcp', Listen => 1, Reuse => 1)
	 or $RSSCache::Log->error(__PACKAGE__."->listen(): Could not create socket on port $port: $!") and return;
	
	$RSSCache::Log->notice(__PACKAGE__."->listen(): Listening on port $port...");
	while (my $socket = $listening_socket->accept()) {
		$RSSCache::Log->debug(__PACKAGE__."->listen(): Incoming connection from peer " . $socket->peerhost() . ". Starting new thread...");
		my $thread = threads->new(\&handle_connection, $socket);
		$thread->detach();
	}
	close($listening_socket);
}
# /listen

#handles one feed request connection.
#will parse the HTTP request and send a proper response.
sub handle_connection {
	my ($connection) = @_;
	
	$RSSCache::Log->debug(__PACKAGE__."->handle_connection(): New thread for peer " . $connection->peerhost() . " started.");
	
	#add input to the parser until a complete HTTP header has been received
	my $parser = HTTP::Parser->new();
	while(defined (my $input = <$connection>)) {
		last if $parser->add($input) == HTTP_HEADER_COMPLETE;
	}
	
	#process request.
	#create HTTP response
	my $encoding = $RSSCache::Settings->get('server/encoding');
	my $response = HTTP::Response->new();
	$response->protocol('HTTP/1.1');
	if (my $request = $parser->request()) {
		my $url = $request->uri();
		#remove leading /'s
		$url =~ s/^\/*//;
		$RSSCache::Log->debug(__PACKAGE__."->handle_connection(): Peer " . $connection->peerhost() . " requested URL '$url'");
		#ignore the favicon
		if ($url eq 'favicon.ico') {
			$response->code(404);
			$response->header('Content-Type' => "text/plain; charset=$encoding");
			$response->content("Not found.");
		} elsif ($url eq 'generate_cid') {
			$response->code(200);
			$response->header('Content-Type' => "text/html; charset=$encoding");
			my $cid = $RSSCache::Backend->generate_client_id();
			my $text = "<html>\n<head><title>Your new client id</title></head>\n<body>\n<p>Your new client id is:</p>\n<p><strong>$cid</strong></p>\n<p>You may how get the feeds like this:</p>\n<p><strong>http://this.server:" . $RSSCache::Settings->get('server/port') . "/cid_$cid/url.to/your/feed</strong></p>\n<p>and you should only receive items, that you havn't downloaded yet.</p></body></html>";
			$response->content(encode($encoding, $text));
		} else {
			#client id. defaults to 0
			my $cid;
			if ($url =~ /^cid_([a-z0-9]*)\/(.+)$/i) {
				#using a client id
				$cid = $1;
				$url = $2;
			}
			$url = "http://$url" unless $url =~ /^https?:\/\//i;
			my $feed = $RSSCache::Backend->feed($url, $cid);
			$cid ||= 'no_cid';
			if (defined $feed) {
				$response->code(200);
				$response->header('Content-Type' => "text/xml; charset=$encoding");
				$response->content(encode($encoding, $feed));
				$RSSCache::Log->debug(__PACKAGE__."->handle_connection(): Delivered feed '$url' for for peer " . $connection->peerhost() . " with client id '$cid'");
			} else {
				$RSSCache::Log->error(__PACKAGE__."->handle_connection(): Couldn't deliver feed '$url' for for peer " . $connection->peerhost());
				$response->code(500);
				$response->header('Content-Type' => "text/plain; charset=$encoding");
				$response->content("Internal Server Error.\nCould not deliver feed '$url'.\nEither the feed's URL is invalid, the server is not reachable or an internal error occured.");
			}
		}
	} else {
		$RSSCache::Log->error(__PACKAGE__."->handle_connection(): Peer " . $connection->peerhost() . " has sent an invalid request");
		$response->code(400);
		$response->header('Content-Type' => "text/plain; charset=$encoding");
		$response->content("Bad Request");
	}
	print $connection $response->as_string();
	close $connection;
}
# /handle_connection

=head1 AUTHOR

Copyright 2006 Thomas Wittek (mail at gedankenkonstrukt dot de). All rights reserved. 

=head1 LICENsE

This document is free software.
It is distributed under the same terms as Perl itself.

=head1 SEE ALSO

L<RSSCache>

=cut

1;
