#! perl

=head1 NAME

RSSCache::Poller - Downloads the feeds

=head1 SYNOPSIS

	$RSSCache::Poller = RSSCache::Poller->new();

=head1 DESCRIPTION

This module will poll one specified or all cached feeds and pass them to the
backend module.

=cut

package RSSCache::Poller;

use strict;
use warnings;

use threads;
use Thread::Queue;
use LWP::UserAgent;
use Time::HiRes qw/time/;

=head1 METHODS

=head2 new

Constructor.

=cut
sub new {
	my ($class) = @_;

	#set default settings
	$RSSCache::Settings->set('poller/max_threads', 5) unless defined $RSSCache::Settings->get('poller/max_threads');
	
	#create new lwp user agent
	my $ua = LWP::UserAgent->new();
	$ua->agent("$0/$RSSCache::VERSION " . $ua->agent());
	$ua->timeout(30); #30 seconds instead of 180 seconds

	return bless { ua => $ua }, $class;
}
# /new

=head2 poll

Downloads the specified feeds and passes them to the backend module that should
update the cache.

This method will use multiple threads to download the feeds.

=head3 Parameters:

=over

=item * @feeds - The URLs of the feeds to download. If no feeds are specified
all cached feeds will be downloaded.

=back

=cut
sub poll {
	my ($self, @feeds) = @_;
	
	#poll all feeds if no feed is specified
	@feeds = $RSSCache::Backend->feeds() unless @feeds;
	
	#enqueue data to process
	$self->{queue} = Thread::Queue->new();
	$self->{queue}->enqueue(@feeds);
	
	#process queue with 5 threads
	my $max_threads = $RSSCache::Settings->get('poller/max_threads');
	$max_threads = @feeds if @feeds < $max_threads;
	$RSSCache::Log->notice(__PACKAGE__."->poll(): Polling " . @feeds . (@feeds == 1 ? " feed" : " feeds"). " with $max_threads threads.");
	my @threads = ( map { threads->new(\&_start_poll_thread, $self) } (1 .. $max_threads) );
	map { $_->join() } @threads;
	
	$RSSCache::Log->notice(__PACKAGE__."->poll(): Finished.");
	return 1;
}
# /poll

#start one downloader thread that will download feeds as long as there are some
#on the queue
sub _start_poll_thread {
	my ($self) = @_;
	
	while (my $url = $self->{queue}->dequeue_nb()) {
		$self->poll_feed($url);
	}
}

=head2 poll_feed

Downloads one specified feed. Won't start a new thread.

=head3 Parameters:

=over

=item * $url - The URLs of the feed to download.

=back

=cut
sub poll_feed {
	my ($self, $url) = @_;
	
	if ($url) {
		$url = "http://$url" unless $url =~ /^https?:\/\//i;
		$RSSCache::Log->notice(__PACKAGE__."->poll_feed(): Downloading feed $url");
		
		#send request
		my $start = time();
		my $result = $self->{ua}->get($url);
		my $duration = time() - $start;
		
		#request successful?
		if ($result->is_success()) {
			$RSSCache::Log->notice(__PACKAGE__.sprintf("->poll_feed(): $url downloaded in %.4f seconds.", $duration));
			$RSSCache::Backend->add_or_update_feed($url, $result->decoded_content());
			return 1;
		} else {
			$RSSCache::Log->error(__PACKAGE__."->poll_feed(): Couldn't download $url. ".$result->status_line());
			$RSSCache::Backend->feed_error($url);
			return undef;
		}
	} else {
		$RSSCache::Log->warning(__PACKAGE__."->poll_feed(): No URL specified.");
		return undef;
	}
}

=head1 AUTHOR

Copyright 2006 Thomas Wittek (mail at gedankenkonstrukt dot de). All rights reserved. 

=head1 LICENsE

This document is free software.
It is distributed under the same terms as Perl itself.

=head1 SEE ALSO

L<RSSCache>, L<RSSCache::Backend>

=cut

1;
