#! perl

=head1 NAME

RSSCache::Scheduler - Scheduler for regularly executed tasks

=head1 SYNOPSIS

	my $scheduler = RSSCache::Scheduler->new();

=head1 DESCRIPTION

This module will start a new thread which checks at every minute if a task has to
be executed.

Currently there is only one available task (poll) which has to be configured in
your settings file, where the configuration is explained.

=cut

package RSSCache::Scheduler;

use strict;
use warnings;

use threads;
use Time::HiRes qw/time sleep/;

use RSSCache::Poller;

=head1 METHODS

=head2 new

Constructor.

Starts the scheduler thread.

=cut
sub new {
	my ($class) = @_;

	#create object
	my $self = bless {}, $class;
	
	#set default settings
	$RSSCache::Settings->set('scheduler/task/poll',  '*-*-*-*-00') unless defined $RSSCache::Settings->get('scheduler/task/poll');
	$RSSCache::Settings->set('scheduler/task/store', '*-*-*-00-00') unless defined $RSSCache::Settings->get('scheduler/task/store');

	#the code for each task should be relatively short/quick or be executed in a
	#separate thread to avoid getting the scheduler out of sync.
	$self->{tasks} = {
		poll  => sub { $RSSCache::Poller->poll() },
		store => sub { $RSSCache::Backend->store_data() }
	};
	
	#create, start and return thread
	return threads->new(\&start, $self);
}
# /new

#the actual scheduler thread
sub start {
	my ($self) = @_;
	
	while (1) {
		#check if there are tasks scheduled for this minute
		$RSSCache::Log->debug(__PACKAGE__.": checking for scheduled tasks...");
		foreach my $task (keys %{$self->{tasks}}) {
			my $task_date = $RSSCache::Settings->get("scheduler/task/$task");
			if (defined $task_date) {
				#convert wildcards into regexp
				$task_date =~ s/\*/\\d\+/go;
				#generate current date formatted as YYYY-MM-DD-HH-MM
				my (undef, $min, $hour, $day, $mon, $year) = localtime time;
				$mon++; $year += 1900;
				my $now = sprintf '%04d-%02d-%02d-%02d-%02d', $year, $mon, $day, $hour, $min;
				#execute the task if the date matches the current date
				$RSSCache::Log->debug(__PACKAGE__.": checking $now =~ /$task_date/");
				if ($now =~ /$task_date/) {
					$RSSCache::Log->notice(__PACKAGE__.": Executing task $task");
					&{$self->{tasks}->{$task}};
				}
			}
		}
		#sleep until the next full minute
		sleep (60 - time % 60 - (time - int(time)));
	}
}
# /start

=head1 AUTHOR

Copyright 2006 Thomas Wittek (mail at gedankenkonstrukt dot de). All rights reserved. 

=head1 LICENsE

This document is free software.
It is distributed under the same terms as Perl itself.

=head1 SEE ALSO

L<RSSCache>

=cut

1;
